/*GR-KURUMI Sketch Template Version: V2.01*/
#include <Arduino.h>

// Pin 6,9,10 are assigned to RGB LEDs.
#define LED_RED2    6 // HIGH active
#define LED_GREEN2  9 // HIGH active
#define LED_BLUE2  10 // HIGH active

//Pin assign
#define CS_RANGE 11
#define CS_CLOCK 8
#define CS_GATE 7
#define CS_DOUT 4
#define CS_LED 12

#define PIN_SPKR 3   //スピーカーR
#define PIN_SPKL 5   //スピーカーL
#define PIN_ACCX A2  //加速度X軸
#define PIN_ACCY A5  //加速度Y軸

//color sensor delay
#define DELAY_T1 delayMicroseconds(5); //5usec
#define DELAY_T2 delayMicroseconds(5); //5usec
#define DELAY_T3 delayMicroseconds(5); //5usec
#define DELAY_T4 delay(2); //2msec
#define DELAY_TW delayMicroseconds(1); //1usec
#define DELAY_TG delay(10); //1000mse

//color no.
#define COLOR_RED         0
#define COLOR_ORANGE      1
#define COLOR_PINK        2
#define COLOR_YELLOW      3
#define COLOR_YELLOWGREEN 4
#define COLOR_GREEN       5
#define COLOR_BLUE        6
#define COLOR_LIGHTBLUE   7
#define COLOR_WHITE       8

/*グローバル変数*/
char selected_color;//選択した色
int accxSens0, accxSens1;//X軸検出用
int accySens0, accySens1;//Y軸検出用

//カラーデータベース
const float samR[9]={1.000,1.000,0.701,0.639,0.259,0.248,0.123,0.115};
const float samG[9]={0.329,0.587,0.506,1.000,1.000,1.000,0.311,0.423};
const float samB[9]={0.459,0.379,1.000,0.270,0.315,0.805,1.000,1.000};

//音データベース        Do   Re   Mi   Fa  Sol   Ra   Si   Do2 
const int freq_tbl[8]={523, 587, 659, 698, 784, 880, 988, 1047};

/*関数定義*/
void changeLED(char colorNo);
void read_color(int *color_r, int *color_g, int *color_b);
char decision_color(void);
float diffRGB(float r, float g, float b, char colorNo);
char difmin(float dif1, float dif2, float dif3, float dif4);

// the setup routine runs once when you press reset:
void setup() {
  pinMode(LED_RED2, OUTPUT);
  pinMode(LED_GREEN2, OUTPUT);
  pinMode(LED_BLUE2, OUTPUT);

  pinMode(CS_RANGE, OUTPUT);
  pinMode(CS_CLOCK, OUTPUT);
  pinMode(CS_GATE, OUTPUT);
  pinMode(CS_DOUT, INPUT);
  pinMode(CS_LED, OUTPUT);
  digitalWrite(CS_LED, HIGH);//白LEDを消す
  pinMode(PIN_SPKR, OUTPUT);
  pinMode(PIN_SPKL, OUTPUT);

  //起動前delay
  delay(1500);
  //からりん起動音
  tone(PIN_SPKR,freq_tbl[5]); // Ra
  delay(125);
  tone(PIN_SPKR,freq_tbl[4]); // Sol
  delay(125);
  tone(PIN_SPKR,freq_tbl[7]); // Do2
  delay(175);
  noTone(PIN_SPKR);

  //initialize
  digitalWrite(CS_RANGE, LOW);
  digitalWrite(CS_CLOCK, LOW);  
  digitalWrite(CS_GATE, LOW); 
  digitalWrite(CS_LED, LOW); 
  digitalWrite(PIN_SPKR, LOW);
  digitalWrite(PIN_SPKL, LOW);
  digitalWrite(LED_RED2, HIGH);
  digitalWrite(LED_GREEN2, HIGH);
  digitalWrite(LED_BLUE2, HIGH);
 
  accxSens0 = analogRead(PIN_ACCX);
  accySens0 = analogRead(PIN_ACCY);
}

// the loop routine runs over and over again forever:
void loop() {
  char selected_color_tmp = decision_color();

  if(selected_color_tmp < COLOR_WHITE){
    selected_color = selected_color_tmp;
    changeLED(selected_color);
  }
  else {
    int accSens;

    accxSens1 = analogRead(PIN_ACCX);
    accySens1 = analogRead(PIN_ACCY);
    accSens = abs(accxSens1-accxSens0)+abs(accySens1-accySens0);
    
    if(accSens > 150){
      tone(PIN_SPKR, freq_tbl[selected_color]);
      delay(400);
      noTone(PIN_SPKR);
      delay(100);
      accxSens1 = analogRead(PIN_ACCX);
      accySens1 = analogRead(PIN_ACCY);
      accxSens0 = accxSens1;
      accySens0 = accySens1;
    }
    else {
      accxSens0 = accxSens1;
      accySens0 = accySens1;
    }
  }
}

void changeLED(char colorNo){
  int ledR, ledG, ledB;
  
  switch(colorNo){
    case COLOR_RED:  
      ledR=255; ledG=0;   ledB=0;
      break;
    case COLOR_ORANGE:
      ledR=255; ledG=120; ledB=0;
      break;
    case COLOR_PINK:
      ledR=255; ledG=0;   ledB=255;
      break;
    case COLOR_YELLOW: 
      ledR=255; ledG=255; ledB=0;
      break;
    case COLOR_YELLOWGREEN:
      ledR=120; ledG=255; ledB=0;
      break;
    case COLOR_GREEN:
      ledR=0;   ledG=255; ledB=0;
      break;
    case COLOR_BLUE:
      ledR=0;   ledG=0;   ledB=255;
      break;
    case COLOR_LIGHTBLUE:
      ledR=0;   ledG=255; ledB=255;
      break;
    case COLOR_WHITE:
      ledR=255; ledG=255; ledB=255;
      break;
  }

  analogWrite(LED_RED2, ledR);
  analogWrite(LED_GREEN2, ledG);
  analogWrite(LED_BLUE2, ledB);
}


void read_color(int *color_r, int *color_g, int *color_b){
  int rgb_value[3] = {0,0,0};
  
  digitalWrite(CS_RANGE, HIGH); //高感度設定
  DELAY_T4
  digitalWrite(CS_GATE, HIGH);
  DELAY_TG
  digitalWrite(CS_GATE, LOW);
  DELAY_T1
  
  for(int i=0; i<3; i++){
    for(int j=0; j<12; j++){
      digitalWrite(CS_CLOCK, HIGH);
      DELAY_TW
      digitalWrite(CS_CLOCK, LOW); 
      rgb_value[i] |= (digitalRead(CS_DOUT)<<j);
      DELAY_TW
    }
    DELAY_T2
  }
  
  *color_r = rgb_value[0];
  *color_g = rgb_value[1];
  *color_b = rgb_value[2];
}

char decision_color(){
  int color_r=0,     color_g=0,     color_b=0;
  int color_r_ave=0, color_g_ave=0, color_b_ave=0;

  char  difColors[4] = {0,0,0,0};
  float dif[4] = {0,0,0,0};
  float cal_r = 0,cal_g = 0,cal_b = 0;

  for(int i=0; i<4; i++){
    read_color(&color_r, &color_g, &color_b);
    color_r_ave += color_r;
    color_g_ave += color_g;
    color_b_ave += color_b;
  }
  color_r = color_r_ave / 4;
  color_g = color_g_ave / 4;
  color_b = color_b_ave / 4;
  
  if((color_r < 500)&&(color_g < 500)&&(color_b < 500)){
    return COLOR_WHITE;
  }

  if(color_r > color_g){
    if(color_r > color_b){  //REGが最大
      cal_g = (float)color_g/color_r;
      cal_b = (float)color_b/color_r;
      cal_r = 1.0;
      difColors[0] = COLOR_RED;
      difColors[1] = COLOR_ORANGE;
      difColors[2] = COLOR_PINK;
      difColors[3] = COLOR_YELLOW;
    }
    else{                   //BLUEが最大
      cal_r = (float)color_r/color_b;
      cal_g = (float)color_g/color_b;
      cal_b = 1.0;
      difColors[0] = COLOR_PINK;
      difColors[1] = COLOR_GREEN;
      difColors[2] = COLOR_BLUE;
      difColors[3] = COLOR_LIGHTBLUE;
    }
  }
  else{
    if(color_g > color_b){  //GREENが最大
      cal_r = (float)color_r/color_g;
      cal_b = (float)color_b/color_g;
      cal_g = 1.0;
      difColors[0] = COLOR_YELLOW;
      difColors[1] = COLOR_YELLOWGREEN;
      difColors[2] = COLOR_GREEN;
      difColors[3] = COLOR_LIGHTBLUE;		
    }
    else{                   //BLUEが最大
      cal_r = (float)color_r/color_b;
      cal_g = (float)color_g/color_b;
      cal_b = 1.0;
      difColors[0] = COLOR_PINK;
      difColors[1] = COLOR_GREEN;
      difColors[2] = COLOR_BLUE;
      difColors[3] = COLOR_LIGHTBLUE;
    }
  }
  dif[0] = diffRGB(cal_r,cal_g,cal_b,difColors[0]);
  dif[1] = diffRGB(cal_r,cal_g,cal_b,difColors[1]);
  dif[2] = diffRGB(cal_r,cal_g,cal_b,difColors[2]);
  dif[3] = diffRGB(cal_r,cal_g,cal_b,difColors[3]);
        
  return difColors[difmin(dif[0],dif[1],dif[2],dif[3])];  
}

float diffRGB(float r, float g, float b, char colorNo){
  float dif;
  dif  = abs(r-samR[colorNo]);
  dif += abs(g-samG[colorNo]);
  dif += abs(b-samB[colorNo]);
  
  return dif;
}

char difmin(float dif0, float dif1, float dif2, float dif3){
  char  dmin0_1=0, dmin2_3=0, dmin=0;
  float dif0_1=0,  dif2_3=0;
  
  if(dif0<dif1){
    dif0_1=dif0;
    dmin0_1=0;
  }
  else{
    dif0_1=dif1;
    dmin0_1=1;
  }

  if(dif2<dif3){
    dif2_3=dif2;
    dmin2_3=2;
  }
  else{
    dif2_3=dif3;
    dmin2_3=3;
  }

  if(dif0_1<dif2_3){
    dmin=dmin0_1;
  }
  else{
    dmin=dmin2_3;
  }
  
  return dmin;
}
